"""Palettize: Python utility and CLI tool for generating and exporting color map objects."""

__version__ = "0.0.1.dev0"  # Initial development version


def hello() -> str:
    return "Hello from palettize!"
